using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using HPS;

namespace hps_cs_samples
{
    public partial class Sample
    {
        public void Conditional_Styles_And_Includes(WindowKey wk, Canvas canvas, View view, Model model, SegmentKey modelKey)
        {
            /* Sample: Conditional_Styles_And_Includes
	         * This sample will show you how to:
	         * - Set styles only when certain conditions are met
	         * - Include geometry only when certain conditions are met
	         * - Create named styles
	         * - Build complex conditions using logical operators
	         * 
	         * Read more about the topics covered in this sample in our Programming Guide 
	         * - section 4.1.3:		Using Conditional Styles
	         * - section 4.1.2:		Defining and Using Styles */

	        /* This next block of code shows you how to load an HSF file
	         * 1. Create an ImportOptionsKit and set the segment in which you want the file to be loaded
	         * 2. Use the Stream::File::Import method to load the desired file, with the options you specified
	         * 3. Since this operation happens on a separate thread, wait for it to complete before proceeding 
	         * 4. Use a try-catch block to capture any Stream IO exceptions */
            HPS.SegmentKey model1 = HPS.Database.CreateRootSegment();
            HPS.SegmentKey model2 = HPS.Database.CreateRootSegment();
            HPS.Stream.ImportNotifier notifier = new HPS.Stream.ImportNotifier();
	        try
	        {
                HPS.Stream.ImportOptionsKit importOptionsKit = new HPS.Stream.ImportOptionsKit();
		        importOptionsKit.SetSegment(model1);
		        string filename = INPUT_FOLDER + "/bnc.hsf";

                notifier = HPS.Stream.File.Import(filename, importOptionsKit);
		        notifier.Wait();

		        filename = INPUT_FOLDER + "/urn.hsf";
		        importOptionsKit.SetSegment(model2);
                notifier = HPS.Stream.File.Import(filename, importOptionsKit);
		        notifier.Wait();
	        }
            catch (HPS.IOException)
	        {
		        //handle Stream IO errors here
	        }

	        /* The next block of code sets up the scene:
	         * 1. Four segments are created from the modelKey segment
	         * 2. Each of the four sub-segments is given its own subwindow
	         * 3. Since light does not transmit through windows, each subwindow needs a light source
	         * 4. The second model is scaled down so that its size is comparable to the first model */
	        model2.GetModellingMatrixControl().Scale(0.01f, 0.01f, 0.01f).Translate(0, 0.03f, 0);
            HPS.SegmentKey topLeft, topRight, bottomLeft, bottomRight;
	        topLeft = modelKey.Subsegment();
	        topLeft.GetSubwindowControl().SetSubwindow(new HPS.Rectangle(-1, 0, 0, 1));
	        topLeft.InsertDistantLight(new HPS.Vector(1, 1, 0));
	        topRight = modelKey.Subsegment();
	        topRight.GetSubwindowControl().SetSubwindow(new HPS.Rectangle(0, 1, 0, 1));
	        topRight.InsertDistantLight(new HPS.Vector(1, 1, 0));
	        bottomLeft = modelKey.Subsegment();
	        bottomLeft.GetSubwindowControl().SetSubwindow(new HPS.Rectangle(-1, 0, -1, 0));
	        bottomLeft.InsertDistantLight(new HPS.Vector(1, 1, 0));
	        bottomRight = modelKey.Subsegment();
	        bottomRight.GetSubwindowControl().SetSubwindow(new HPS.Rectangle(0, 1, -1, 0));
	        bottomRight.InsertDistantLight(new HPS.Vector(1, 1, 0));

	        /* This block of code defines four named styles
	         * 1. For each style create a segment which contains the attribute for the style
	         * 2. Obtain the key for a portfolio
	         * 3. Define named styles by giving them a name and the segment key that contains the
	         *		attributes for each style */
            HPS.SegmentKey style1, style2, style3, style4;
	        style1 = modelKey.Subsegment();
	        style1.GetVisibilityControl().SetFaces(false).SetEdges(true).SetLights(false);
	        style2 = modelKey.Subsegment();
	        style2.GetLightingAttributeControl().SetInterpolationAlgorithm(Lighting.InterpolationAlgorithm.Phong);
	        style3 = modelKey.Subsegment();
	        style3.GetVisibilityControl().SetEdges(true);
            style3.GetMaterialMappingControl().SetEdgeColor(new HPS.RGBAColor(1, 0, 0, 1));
	        style4 = modelKey.Subsegment();
	        style4.GetVisibilityControl().SetEdges(true);
            style4.GetMaterialMappingControl().SetEdgeColor(new HPS.RGBAColor(0, 0, 1, 1));

            HPS.PortfolioKey pKey;
	        modelKey.GetPortfolioControl().ShowTop(out pKey);
	        pKey.DefineNamedStyle("wireframe", style1);
	        pKey.DefineNamedStyle("phong", style2);
	        pKey.DefineNamedStyle("redEdges", style3);
	        pKey.DefineNamedStyle("blueEdges", style4);

	        /* This block of code shows how to style a segment conditionally
	         * The segment model1 will only use the 'wireframe' style is the condition 'bottom'
	         * evaluates to TRUE
	         * 
	         * If more than one condition evaluates to true, than multiple styles are used */
	        string t = "top";
            string b = "bottom";
            string l = "left";
            string r = "right";
            model1.GetStyleControl().PushNamed("wireframe", new HPS.Condition(b));
            model1.GetStyleControl().PushNamed("phong", new HPS.Condition(t));
            model1.GetStyleControl().PushNamed("redEdges", new HPS.Condition(r));
            model1.GetStyleControl().PushNamed("blueEdges", new HPS.Condition(l));
            model1.IncludeSegment(model2, HPS.Condition.AND(new HPS.Condition(b), new HPS.Condition(r)));

	        /* The next block of code sets multiple conditions on each of the four subwindows
	         * and includes the geometry which has the conditional styles set on it */ 
	        string[] c1 = new string[2];
            string[] c2 = new string[2];
            string[] c3 = new string[2];
            string[] c4 = new string[2];
            c1[0] = t; c1[1] = l;
            c2[0] = t; c2[1] = r;
            c3[0] = b; c3[1] = l;
            c4[0] = b; c4[1] = r;

	        topLeft.IncludeSegment(model1);
	        topLeft.SetConditions(c1);
	        topRight.IncludeSegment(model1);
	        topRight.SetConditions(c2);
	        bottomLeft.IncludeSegment(model1);
	        bottomLeft.SetConditions(c3);
	        bottomRight.IncludeSegment(model1);
	        bottomRight.SetConditions(c4);

	        view.Update();
            HPS.CameraKit cameraKit = new HPS.CameraKit();
            cameraKit.SetPosition(new HPS.Point(-0.04f, 0.11f, 0.05f)).SetTarget(new HPS.Point(0, 0, 0.01f)).SetUpVector(new HPS.Vector(0, 0, 1)).
                SetField(0.075f, 0.076f).SetProjection(HPS.Camera.Projection.Perspective);
	        view.GetSegmentKey().SetCamera(cameraKit);
        }
    }
}
